var _interopRequireDefault=require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports,"__esModule",{value:true});exports.default=exports.PIPES=exports.cleanPlatformBuild=exports.createPlatformBuild=void 0;

var _chalk=_interopRequireDefault(require("chalk"));
var _path=_interopRequireDefault(require("path"));
var _fs=_interopRequireDefault(require("fs"));
var _common=require("../common");









var _fileutils=require("../systemTools/fileutils");
var _buildHooks=require("../projectTools/buildHooks");
var _constants=require("../constants");

var CONFIGURE='configure';
var UPDATE='update';
var LIST='list';
var ADD='add';
var REMOVE='remove';
var EJECT='eject';
var CONNECT='connect';

var PIPES={
PLATFORM_CONFIGURE_BEFORE:'platform:configure:before',
PLATFORM_CONFIGURE_AFTER:'platform:configure:after',
PLATFORM_UPDATE_BEFORE:'platform:update:before',
PLATFORM_UPDATE_AFTER:'platform:update:after',
PLATFORM_LIST_BEFORE:'platform:list:before',
PLATFORM_LIST_AFTER:'platform:list:after',
PLATFORM_ADD_BEFORE:'platform:add:before',
PLATFORM_ADD_AFTER:'platform:add:after',
PLATFORM_REMOVE_BEFORE:'platform:remove:before',
PLATFORM_REMOVE_AFTER:'platform:remove:after',
PLATFORM_EJECT_BEFORE:'platform:eject:before',
PLATFORM_EJECT_AFTER:'platform:ejecct:after',
PLATFORM_CONNECT_BEFORE:'platform:connect:before',
PLATFORM_CONNECT_AFTER:'platform:connect:after'};exports.PIPES=PIPES;






var run=function run(c){
(0,_common.logTask)('run');

switch(c.subCommand){
case CONFIGURE:
return _runCreatePlatforms(c);
case EJECT:
return _runEjectPlatforms(c);
case CONNECT:
return _runConnectPlatforms(c);
case LIST:
return _runListPlatforms(c);











default:
return Promise.reject("Sub-Command "+c.subCommand+" not supported");}

};






var _runListPlatforms=function _runListPlatforms(c){return new Promise(function(resolve,reject){
var opts=_genPlatOptions(c);
console.log("\n"+opts.asString);
resolve();
});};

var _runCreatePlatforms=function _runCreatePlatforms(c){return new Promise(function(resolve,reject){
var p=c.program.platform||'all';
(0,_common.logTask)("_runCreatePlatforms:"+p);

(0,_buildHooks.executePipe)(c,PIPES.PLATFORM_CONFIGURE_BEFORE).
then(function(){return cleanPlatformBuild(c,p);}).
then(function(){return _runCleanPlaformAssets(c);}).
then(function(){return _runCopyPlatforms(c,p);}).
then(function(){return(0,_buildHooks.executePipe)(c,PIPES.PLATFORM_CONFIGURE_AFTER);}).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
});};

var _runEjectPlatforms=function _runEjectPlatforms(c){return new Promise(function(resolve){
(0,_common.logTask)('_runEjectPlatforms');

var opts=_genPlatOptions(c);

(0,_common.askQuestion)("This will copy platformTemplates folders from ReNative managed directly to your project. Select platforms you would like to eject comma separated\n"+opts.asString).
then(function(v){return opts.pick(v);}).
then(function(v){
(0,_common.finishQuestion)();


var ptfn='platformTemplates';
var rptf=c.paths.rnvPlatformTemplatesFolder;
var prf=c.paths.projectRootFolder;

var WEB_BASED_PLATFORMS=[];
var copyShared=false;

opts.selectedOptions.forEach(function(v){
if(_constants.PLATFORMS[v].usesSharedConfig){
copyShared=true;
}

(0,_fileutils.copyFolderContentsRecursiveSync)(_path.default.join(rptf,v),_path.default.join(prf,ptfn,v));

if(copyShared){
(0,_fileutils.copyFolderContentsRecursiveSync)(_path.default.join(rptf,'_shared'),_path.default.join(prf,ptfn,'_shared'));
}

c.files.projectConfig.platformTemplatesFolders=c.files.projectConfig.platformTemplatesFolders||{};
c.files.projectConfig.platformTemplatesFolders[v]="./"+ptfn;

(0,_fileutils.writeObjectSync)(c.paths.projectConfigPath,c.files.projectConfig);
});
(0,_common.logSuccess)(
_chalk.default.white(opts.selectedOptions.join(','))+" platform templates are located in "+_chalk.default.white(
c.files.projectConfig.platformTemplatesFolders[opts.selectedOptions[0]])+" now. You can edit them directly!");


resolve();
});
});};

var _genPlatOptions=function _genPlatOptions(c){
var opts=(0,_common.generateOptions)(c.files.projectConfig.defaultProjectConfigs.supportedPlatforms,true,null,function(i,obj,mapping,defaultVal){
var isEjected=c.paths.platformTemplatesFolders[obj].includes(c.paths.rnvPlatformTemplatesFolder)?_chalk.default.green('(connected)'):_chalk.default.yellow('(ejected)');
return"-["+_chalk.default.white(i+1)+"] "+_chalk.default.white(defaultVal)+" - "+isEjected+" \n";
});
return opts;
};

var _runConnectPlatforms=function _runConnectPlatforms(c){return new Promise(function(resolve){
(0,_common.logTask)('_runConnectPlatforms');

var opts=_genPlatOptions(c);

(0,_common.askQuestion)("This will point platformTemplates folders from your local project to ReNative managed one. Select platforms you would like to connect comma separated\n"+opts.asString).
then(function(v){return opts.pick(v);}).
then(function(){
(0,_common.finishQuestion)();
opts.selectedOptions.forEach(function(v){
var ptfn='platformTemplates';

if(!c.files.projectConfig.platformTemplatesFolders)c.files.projectConfig.platformTemplatesFolders={};

c.files.projectConfig.platformTemplatesFolders[v]="RNV_HOME/"+ptfn;

(0,_fileutils.writeObjectSync)(c.paths.projectConfigPath,c.files.projectConfig);
});
(0,_common.logSuccess)(
_chalk.default.white(opts.selectedOptions.join(','))+" now using ReNative platformTemplates located in "+_chalk.default.white(c.paths.rnvPlatformTemplatesFolder)+" now!");

resolve();
});
});};

var _addPlatform=function _addPlatform(platform,program,process){return new Promise(function(resolve,reject){
if(!(0,_common.isPlatformSupportedSync)(platform,resolve))return;

(0,_common.getConfig)().then(function(v){
_runAddPlatform().
then(function(){return resolve();}).
catch(function(e){return reject(e);});
});
});};

var _removePlatform=function _removePlatform(platform,program,process){return new Promise(function(resolve,reject){
if(!(0,_common.isPlatformSupportedSync)(platform,resolve))return;
console.log('REMOVE_PLATFORM: ',platform);
resolve();
});};

var _runCleanPlaformAssets=function _runCleanPlaformAssets(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('_runCleanPlaformAssets');

(0,_fileutils.cleanFolder)(c.paths.platformAssetsFolder).then(function(){
resolve();
});
});};

var _runCopyPlatforms=function _runCopyPlatforms(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)("_runCopyPlatforms:"+platform);
var copyPlatformTasks=[];
if(platform==='all'){
for(var k in c.files.appConfigFile.platforms){
if((0,_common.isPlatformSupportedSync)(k)){
var ptPath=_path.default.join(c.paths.platformTemplatesFolders[k],""+k);
var pPath=_path.default.join(c.paths.platformBuildsFolder,c.appId+"_"+k);
copyPlatformTasks.push((0,_fileutils.copyFolderContentsRecursiveSync)(ptPath,pPath));
}
}
}else if((0,_common.isPlatformSupportedSync)(platform)){
var _ptPath=_path.default.join(c.paths.platformTemplatesFolders[platform],""+platform);
var _pPath=_path.default.join(c.paths.platformBuildsFolder,c.appId+"_"+platform);
copyPlatformTasks.push((0,_fileutils.copyFolderContentsRecursiveSync)(_ptPath,_pPath));
}else{
logWarning("Your platform "+_chalk.default.white(platform)+" config is not present. Check "+_chalk.default.white(c.paths.appConfigPath));
}

Promise.all(copyPlatformTasks).then(function(values){
resolve();
});
});};

var cleanPlatformBuild=function cleanPlatformBuild(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)("cleanPlatformBuild:"+platform);

var cleanTasks=[];

if(platform==='all'){
for(var k in c.files.appConfigFile.platforms){
if((0,_common.isPlatformSupportedSync)(k)){
var pPath=_path.default.join(c.paths.platformBuildsFolder,c.appId+"_"+k);
cleanTasks.push((0,_fileutils.cleanFolder)(pPath));
}
}
}else if((0,_common.isPlatformSupportedSync)(platform)){
var _pPath2=_path.default.join(c.paths.platformBuildsFolder,c.appId+"_"+platform);
cleanTasks.push((0,_fileutils.cleanFolder)(_pPath2));
}

Promise.all(cleanTasks).then(function(values){
resolve();
});
});};exports.cleanPlatformBuild=cleanPlatformBuild;

var _runAddPlatform=function _runAddPlatform(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('runAddPlatform');
resolve();
});};

var createPlatformBuild=function createPlatformBuild(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)("createPlatformBuild:"+platform);

if(!(0,_common.isPlatformSupportedSync)(platform,null,reject))return;

var pPath=_path.default.join(c.paths.platformBuildsFolder,c.appId+"_"+platform);
var ptPath=_path.default.join(c.paths.platformTemplatesFolders[platform],""+platform);
(0,_fileutils.copyFolderContentsRecursiveSync)(ptPath,pPath,false,[_path.default.join(ptPath,'_privateConfig')]);

resolve();
});};exports.createPlatformBuild=createPlatformBuild;var _default=



run;exports.default=_default;