var _interopRequireDefault=require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports,"__esModule",{value:true});exports.launchTizenSimulator=launchTizenSimulator;exports.configureTizenGlobal=exports.buildTizenProject=exports.runTizen=exports.addDevelopTizenCertificate=exports.createDevelopTizenCertificate=exports.configureTizenProject=exports.copyTizenAssets=void 0;var _regenerator=_interopRequireDefault(require("@babel/runtime/regenerator"));
var _path=_interopRequireDefault(require("path"));
var _fs=_interopRequireDefault(require("fs"));
var _chalk=_interopRequireDefault(require("chalk"));
var _child_process=_interopRequireDefault(require("child_process"));
var _inquirer=_interopRequireDefault(require("inquirer"));
var _net=_interopRequireDefault(require("net"));

var _exec=require("../systemTools/exec");
var _constants=require("../constants");
var _common=require("../common");
















var _fileutils=require("../systemTools/fileutils");
var _web=require("./web");

var configureTizenGlobal=function configureTizenGlobal(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('configureTizenGlobal');


var tizenAuthorCert=_path.default.join(c.paths.globalConfigFolder,'tizen_author.p12');
if(_fs.default.existsSync(tizenAuthorCert)){
console.log('tizen_author.p12 file exists!');
resolve();
}else{
console.log('tizen_author.p12 file missing! Creating one for you...');
createDevelopTizenCertificate(c).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
}

});};exports.configureTizenGlobal=configureTizenGlobal;

function launchTizenSimulator(c,name){
(0,_common.logTask)("launchTizenSimulator:"+name);

if(name){
return(0,_exec.execCLI)(c,_common.CLI_TIZEN_EMULATOR,"launch --name "+name);
}
return Promise.reject('No simulator -t target name specified!');
}

var copyTizenAssets=function copyTizenAssets(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)('copyTizenAssets');
if(!(0,_common.isPlatformActive)(c,platform,resolve))return;

var sourcePath=_path.default.join(c.paths.appConfigFolder,'assets',platform);
var destPath=_path.default.join((0,_common.getAppFolder)(c,platform));

(0,_fileutils.copyFolderContentsRecursiveSync)(sourcePath,destPath);
resolve();
});};exports.copyTizenAssets=copyTizenAssets;

var createDevelopTizenCertificate=function createDevelopTizenCertificate(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('createDevelopTizenCertificate');

(0,_exec.execCLI)(c,_common.CLI_TIZEN,"certificate -- "+c.paths.globalConfigFolder+" -a rnv -f tizen_author -p 1234").
then(function(){return addDevelopTizenCertificate(c);}).
then(function(){return resolve();}).
catch(function(e){
(0,_common.logError)(e);
resolve();
});
});};exports.createDevelopTizenCertificate=createDevelopTizenCertificate;

var addDevelopTizenCertificate=function addDevelopTizenCertificate(c){return new Promise(function(resolve){
(0,_common.logTask)('addDevelopTizenCertificate');

(0,_exec.execCLI)(c,_common.CLI_TIZEN,"security-profiles add -n RNVanillaCert -a "+_path.default.join(c.paths.globalConfigFolder,'tizen_author.p12')+" -p 1234").
then(function(){return resolve();}).
catch(function(e){
(0,_common.logError)(e);
resolve();
});
});};exports.addDevelopTizenCertificate=addDevelopTizenCertificate;

var getDeviceID=function getDeviceID(c,target){var device,connectResponse,devicesList,lines,devices,deviceID;return _regenerator.default.async(function getDeviceID$(_context){while(1){switch(_context.prev=_context.next){case 0:
device=c.program.device;if(!
device){_context.next=7;break;}_context.next=4;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_common.CLI_SDB_TIZEN,"connect "+target,_common.logTask));case 4:connectResponse=_context.sent;if(!
connectResponse.includes('failed to connect to remote target')){_context.next=7;break;}throw new Error(connectResponse);case 7:_context.next=9;return _regenerator.default.awrap(


(0,_exec.execCLI)(c,_common.CLI_SDB_TIZEN,'devices',_common.logTask));case 9:devicesList=_context.sent;if(!
devicesList.includes(target)){_context.next=16;break;}
lines=devicesList.trim().split(/\r?\n/);
devices=lines.filter(function(line){return line.includes(target);});

if(devices.length>1){

}

deviceID=devices[0].split('device')[1].trim();return _context.abrupt("return",
deviceID);case 16:throw"No device matching "+

target+" could be found.";case 17:case"end":return _context.stop();}}});};


var waitForEmulatorToBeReady=function waitForEmulatorToBeReady(c,emulator){return new Promise(function(resolve){
var attempts=0;
var maxAttempts=10;
var poll=setInterval(function(){
try{
var devicesList=_child_process.default.execSync(c.cli[_common.CLI_SDB_TIZEN]+" devices").toString();
var lines=devicesList.trim().split(/\r?\n/);
var devices=lines.filter(function(line){return line.includes(emulator)&&line.includes('device');});
if(devices.length>0){
clearInterval(poll);
(0,_common.logDebug)('waitForEmulatorToBeReady - boot complete');
resolve(true);
}else{
attempts++;
console.log("Checking if emulator has booted up: attempt "+attempts+"/"+maxAttempts);
if(attempts===maxAttempts){
clearInterval(poll);
throw new Error('Can\'t connect to the running emulator. Try restarting it.');
}
}
}catch(e){
console.log("Checking if emulator has booted up: attempt "+attempts+"/"+maxAttempts);
attempts++;
if(attempts>maxAttempts){
clearInterval(poll);
throw new Error('Can\'t connect to the running emulator. Try restarting it.');
}
}
},2000);
});};

var getRunningDevices=function getRunningDevices(c){var devicesList,lines,devices;return _regenerator.default.async(function getRunningDevices$(_context2){while(1){switch(_context2.prev=_context2.next){case 0:
devicesList=_child_process.default.execSync(c.cli[_common.CLI_SDB_TIZEN]+" devices").toString();
lines=devicesList.trim().split(/\r?\n/);
devices=[];

if(lines.length>1){
lines.forEach(function(line){
if(!line.includes('List of devices')){
var words=line.replace(/\t/g,'').split('    ');
if(words.length>=3){
devices.push({
name:words[0].trim(),
type:words[1].trim(),
id:words[2].trim()});

}
}
});
}return _context2.abrupt("return",

devices);case 5:case"end":return _context2.stop();}}});};


var composeDevicesString=function composeDevicesString(devices){return devices.map(function(device){return{key:device.id,name:device.name,value:device.id};});};

var runTizen=function runTizen(c,platform,target){var platformConfig,_c$program,hosted,device,isHosted,tDir,tBuild,tOut,tId,gwt,certProfile,deviceID,askForEmulator,continueLaunching,isTargetSpecified,devices,targetDevice,choices,_ref2,chosenEmulator;return _regenerator.default.async(function runTizen$(_context5){while(1){switch(_context5.prev=_context5.next){case 0:
(0,_common.logTask)("runTizen:"+platform+":"+target);

platformConfig=c.files.appConfigFile.platforms[platform];_c$program=
c.program,hosted=_c$program.hosted,device=_c$program.device;

isHosted=hosted||!(0,_common.getConfigProp)(c,platform,'bundleAssets');if(

platformConfig){_context5.next=6;break;}throw(
new Error("runTizen: "+_chalk.default.blue(platform)+" not defined in your "+_chalk.default.white(c.paths.appConfigPath)));case 6:if(

platformConfig.appName){_context5.next=8;break;}throw(
new Error("runTizen: "+_chalk.default.blue(platform)+".appName not defined in your "+_chalk.default.white(c.paths.appConfigPath)));case 8:


tDir=(0,_common.getAppFolder)(c,platform);
tBuild=_path.default.join(tDir,'build');
tOut=_path.default.join(tDir,'output');
tId=platformConfig.id;
gwt=platformConfig.appName+".wgt";
certProfile=platformConfig.certificateProfile;




askForEmulator=function askForEmulator(){var _ref,startEmulator,defaultTarget;return _regenerator.default.async(function askForEmulator$(_context3){while(1){switch(_context3.prev=_context3.next){case 0:_context3.next=2;return _regenerator.default.awrap(
_inquirer.default.prompt([{
name:'startEmulator',
type:'confirm',
message:"Could not find or connect to the specified target ("+target+"). Would you like to start an emulator?"}]));case 2:_ref=_context3.sent;startEmulator=_ref.startEmulator;if(!


startEmulator){_context3.next=33;break;}
defaultTarget=c.files.globalConfig.defaultTargets[platform];_context3.prev=6;_context3.next=9;return _regenerator.default.awrap(

launchTizenSimulator(c,defaultTarget));case 9:
deviceID=defaultTarget;_context3.next=12;return _regenerator.default.awrap(
waitForEmulatorToBeReady(c,defaultTarget));case 12:return _context3.abrupt("return",
continueLaunching());case 15:_context3.prev=15;_context3.t0=_context3["catch"](6);

(0,_common.logDebug)("askForEmulator:ERRROR: "+_context3.t0);_context3.prev=18;_context3.next=21;return _regenerator.default.awrap(

(0,_exec.execCLI)(c,_common.CLI_TIZEN_EMULATOR,"create -n "+defaultTarget+" -p tv-samsung-5.0-x86",_common.logTask));case 21:_context3.next=23;return _regenerator.default.awrap(
launchTizenSimulator(c,defaultTarget));case 23:
deviceID=defaultTarget;_context3.next=26;return _regenerator.default.awrap(
waitForEmulatorToBeReady(c,defaultTarget));case 26:return _context3.abrupt("return",
continueLaunching());case 29:_context3.prev=29;_context3.t1=_context3["catch"](18);

(0,_common.logDebug)(_context3.t1);
(0,_common.logError)("Could not find the specified target and could not create the emulator automatically. Please create one and then edit the default target from "+c.paths.globalConfigFolder+"/"+_constants.RNV_GLOBAL_CONFIG_NAME);case 33:case"end":return _context3.stop();}}},null,null,[[6,15],[18,29]]);};





continueLaunching=function continueLaunching(){var hasDevice,packageID;return _regenerator.default.async(function continueLaunching$(_context4){while(1){switch(_context4.prev=_context4.next){case 0:
hasDevice=false;_context4.t0=

!isHosted;if(!_context4.t0){_context4.next=5;break;}_context4.next=5;return _regenerator.default.awrap((0,_web.buildWeb)(c,platform));case 5:_context4.next=7;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_common.CLI_TIZEN,"build-web -- "+tDir+" -out "+tBuild,_common.logTask));case 7:_context4.next=9;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_common.CLI_TIZEN,"package -- "+tBuild+" -s "+certProfile+" -t wgt -o "+tOut,_common.logTask));case 9:_context4.prev=9;_context4.next=12;return _regenerator.default.awrap(


(0,_exec.execCLI)(c,_common.CLI_TIZEN,"uninstall -p "+tId+" -t "+deviceID,_common.logTask));case 12:
hasDevice=true;_context4.next=23;break;case 15:_context4.prev=15;_context4.t1=_context4["catch"](9);if(!(

_context4.t1&&_context4.t1.includes&&_context4.t1.includes('No device matching'))){_context4.next=23;break;}_context4.next=20;return _regenerator.default.awrap(
launchTizenSimulator(c,target));case 20:_context4.next=22;return _regenerator.default.awrap(
waitForEmulatorToBeReady(c,target));case 22:hasDevice=_context4.sent;case 23:_context4.prev=23;_context4.next=26;return _regenerator.default.awrap(



(0,_exec.execCLI)(c,_common.CLI_TIZEN,"install -- "+tOut+" -n "+gwt+" -t "+deviceID,_common.logTask));case 26:
hasDevice=true;_context4.next=38;break;case 29:_context4.prev=29;_context4.t2=_context4["catch"](23);

(0,_common.logError)(_context4.t2);
(0,_common.logWarning)("Looks like there is no emulator or device connected! Let's try to launch it. \""+
_chalk.default.white.bold("rnv target launch -p "+
platform+" -t "+target)+"\"");_context4.next=35;return _regenerator.default.awrap(



launchTizenSimulator(c,target));case 35:_context4.next=37;return _regenerator.default.awrap(
waitForEmulatorToBeReady(c,target));case 37:hasDevice=_context4.sent;case 38:if(!(


platform!=='tizenwatch'&&platform!=='tizenmobile'&&hasDevice)){_context4.next=43;break;}_context4.next=41;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_common.CLI_TIZEN,"run -p "+tId+" -t "+deviceID,_common.logTask));case 41:_context4.next=47;break;case 43:if(!(
(platform==='tizenwatch'||platform==='tizenmobile')&&hasDevice)){_context4.next=47;break;}
packageID=tId.split('.');_context4.next=47;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_common.CLI_TIZEN,"run -p "+packageID[0]+" -t "+deviceID,_common.logTask));case 47:return _context4.abrupt("return",

true);case 48:case"end":return _context4.stop();}}},null,null,[[9,15],[23,29]]);};



isTargetSpecified=c.program.target;_context5.next=19;return _regenerator.default.awrap(


getRunningDevices(c));case 19:devices=_context5.sent;if(!

isTargetSpecified){_context5.next=45;break;}if(!

_net.default.isIP(target)){_context5.next=26;break;}_context5.next=24;return _regenerator.default.awrap(
getDeviceID(c,target));case 24:deviceID=_context5.sent;return _context5.abrupt("return",
continueLaunching());case 26:if(!(


devices.length>0)){_context5.next=31;break;}
targetDevice=devices.find(function(device){return device.id===target||device.name===target;});if(!
targetDevice){_context5.next=31;break;}
deviceID=targetDevice.id;return _context5.abrupt("return",
continueLaunching());case 31:_context5.prev=31;_context5.next=34;return _regenerator.default.awrap(




launchTizenSimulator(c,target));case 34:_context5.next=36;return _regenerator.default.awrap(
waitForEmulatorToBeReady(c,target));case 36:
deviceID=target;return _context5.abrupt("return",
continueLaunching());case 40:_context5.prev=40;_context5.t0=_context5["catch"](31);return _context5.abrupt("return",

askForEmulator());case 43:_context5.next=57;break;case 45:if(!(


devices.length===1)){_context5.next=48;break;}
deviceID=devices[0].id;return _context5.abrupt("return",
continueLaunching());case 48:if(!(
devices.length>1)){_context5.next=56;break;}
choices=composeDevicesString(devices);_context5.next=52;return _regenerator.default.awrap(
_inquirer.default.prompt([{
name:'chosenEmulator',
type:'list',
message:'On what emulator would you like to run the app?',
choices:choices}]));case 52:_ref2=_context5.sent;chosenEmulator=_ref2.chosenEmulator;

deviceID=chosenEmulator;return _context5.abrupt("return",
continueLaunching());case 56:return _context5.abrupt("return",

askForEmulator());case 57:case"end":return _context5.stop();}}},null,null,[[31,40]]);};exports.runTizen=runTizen;



var buildTizenProject=function buildTizenProject(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)("buildTizenProject:"+platform);

var platformConfig=c.files.appConfigFile.platforms[platform];
var tDir=(0,_common.getAppFolder)(c,platform);

var tOut=_path.default.join(tDir,'output');
var tBuild=_path.default.join(tDir,'build');
var certProfile=platformConfig.certificateProfile;

(0,_web.buildWeb)(c,platform).
then(function(){return(0,_exec.execCLI)(c,_common.CLI_TIZEN,"build-web -- "+tDir+" -out "+tBuild,_common.logTask);}).
then(function(){return(0,_exec.execCLI)(c,_common.CLI_TIZEN,"package -- "+tBuild+" -s "+certProfile+" -t wgt -o "+tOut,_common.logTask);}).
then(function(){
(0,_common.logSuccess)("Your GWT package is located in "+_chalk.default.white(tOut)+" .");
return resolve();
}).
catch(function(e){return reject(e);});
});};exports.buildTizenProject=buildTizenProject;

var configureTizenProject=function configureTizenProject(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)('configureTizenProject');

if(!(0,_common.isPlatformActive)(c,platform,resolve))return;



copyTizenAssets(c,platform).
then(function(){return(0,_common.copyBuildsFolder)(c,platform);}).
then(function(){return configureProject(c,platform);}).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
});};exports.configureTizenProject=configureTizenProject;

var configureProject=function configureProject(c,platform){return new Promise(function(resolve){
(0,_common.logTask)("configureProject:"+platform);

var appFolder=(0,_common.getAppFolder)(c,platform);

var configFile='config.xml';
var p=c.files.appConfigFile.platforms[platform];
(0,_common.writeCleanFile)(_path.default.join((0,_common.getAppTemplateFolder)(c,platform),configFile),_path.default.join(appFolder,configFile),[
{pattern:'{{PACKAGE}}',override:p.package},
{pattern:'{{ID}}',override:p.id},
{pattern:'{{APP_NAME}}',override:p.appName}]);


resolve();
});};