var _interopRequireDefault=require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports,"__esModule",{value:true});exports.listWebOSTargets=exports.buildWebOSProject=exports.runWebOS=exports.configureWebOSProject=exports.copyWebOSAssets=exports.launchWebOSimulator=void 0;var _regenerator=_interopRequireDefault(require("@babel/runtime/regenerator"));var _slicedToArray2=_interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _path=_interopRequireDefault(require("path"));
var _fs=_interopRequireDefault(require("fs"));
var _chalk=_interopRequireDefault(require("chalk"));
var _semver=_interopRequireDefault(require("semver"));
var _inquirer=_interopRequireDefault(require("inquirer"));
var _exec=require("../systemTools/exec");
var _common=require("../common");





















var _constants=require("../constants");












var _fileutils=require("../systemTools/fileutils");
var _web=require("./web");

var isRunningOnWindows=process.platform==='win32';

var launchWebOSimulator=function launchWebOSimulator(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('launchWebOSimulator');

var ePath=_path.default.join(c.files.globalConfig.sdks.WEBOS_SDK,"Emulator/v4.0.0/LG_webOS_TV_Emulator"+(isRunningOnWindows?'.exe':'_RCU.app'));

if(!_fs.default.existsSync(ePath)){
reject("Can't find emulator at path: "+ePath);
return;
}

var childProcess=require('child_process');
childProcess.exec((process.platform=='darwin'?'open':process.platform=='win32'?'start':'xdg-open')+" "+ePath,function(err,stdout,stderr){
if(err){
reject(err);
return;
}
resolve();
});
});};exports.launchWebOSimulator=launchWebOSimulator;

var copyWebOSAssets=function copyWebOSAssets(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)('copyWebOSAssets');
if(!(0,_common.isPlatformActive)(c,platform,resolve))return;

var sourcePath=_path.default.join(c.paths.appConfigFolder,'assets',platform);
var destPath=_path.default.join((0,_common.getAppFolder)(c,platform),'public');

(0,_fileutils.copyFolderContentsRecursiveSync)(sourcePath,destPath);
resolve();
});};exports.copyWebOSAssets=copyWebOSAssets;

var parseDevices=function parseDevices(devicesResponse){
var linesArray=devicesResponse.split('\n').slice(2).map(function(line){return line.trim();});
return linesArray.map(function(line){var _line$split$map$filte=
line.split(' ').map(function(word){return word.trim();}).filter(function(word){return word!=='';}),_line$split$map$filte2=(0,_slicedToArray2.default)(_line$split$map$filte,4),name=_line$split$map$filte2[0],device=_line$split$map$filte2[1],connection=_line$split$map$filte2[2],profile=_line$split$map$filte2[3];
return{
name:name,
device:device,
connection:connection,
profile:profile,
isDevice:!device.includes('127.0.0.1')};

});
};

var installAndLaunchApp=function installAndLaunchApp(c,target,appPath,tId){return _regenerator.default.async(function installAndLaunchApp$(_context){while(1){switch(_context.prev=_context.next){case 0:_context.prev=0;_context.next=3;return _regenerator.default.awrap(

(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_INSTALL,"--device "+target+" "+appPath,_common.logTask));case 3:_context.next=5;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_LAUNCH,"--device "+target+" "+tId,_common.logTask));case 5:_context.next=17;break;case 7:_context.prev=7;_context.t0=_context["catch"](0);if(!(

_context.t0&&_context.t0.toString().includes(_constants.CLI_WEBOS_ARES_INSTALL))){_context.next=16;break;}
(0,_common.logWarning)("Looks like there is no emulator or device connected! Let's try to launch it. \""+
_chalk.default.white.bold(
'rnv target launch -p webos -t emulator')+"\"");_context.next=13;return _regenerator.default.awrap(



launchWebOSimulator(c,target));case 13:
(0,_common.logInfo)("Once simulator is ready run: \""+
_chalk.default.white.bold('rnv run -p webos -t emulator')+"\" again");_context.next=17;break;case 16:throw _context.t0;case 17:case"end":return _context.stop();}}},null,null,[[0,7]]);};







var buildDeviceChoices=function buildDeviceChoices(devices){return devices.map(function(device){return{
key:device.name,name:device.name+" - "+device.device,value:device.name};});};


var listWebOSTargets=function listWebOSTargets(c){var devicesResponse,devices,deviceArray;return _regenerator.default.async(function listWebOSTargets$(_context2){while(1){switch(_context2.prev=_context2.next){case 0:_context2.next=2;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_DEVICE_INFO,'-D'));case 2:devicesResponse=_context2.sent;
devices=parseDevices(devicesResponse);

deviceArray=devices.map(function(device,i){return"-["+(i+1)+"] "+device.name+" | "+device.device;});
console.log(deviceArray.join('\n'));return _context2.abrupt("return",
true);case 7:case"end":return _context2.stop();}}});};exports.listWebOSTargets=listWebOSTargets;


var runWebOS=function runWebOS(c,platform,target){var _c$program,device,hosted,isHosted,tDir,tOut,tSim,configFilePath,cnfg,tId,appPath,devicesResponse,devices,actualDevices,response,newDeviceResponse,dev,actualDev,newDevice,tv,choices,_response;return _regenerator.default.async(function runWebOS$(_context3){while(1){switch(_context3.prev=_context3.next){case 0:
(0,_common.logTask)("runWebOS:"+platform+":"+target);_c$program=

c.program,device=_c$program.device,hosted=_c$program.hosted;

isHosted=hosted||!(0,_common.getConfigProp)(c,platform,'bundleAssets');

tDir=_path.default.join((0,_common.getAppFolder)(c,platform),'public');
tOut=_path.default.join((0,_common.getAppFolder)(c,platform),'output');
tSim=c.program.target||'emulator';
configFilePath=_path.default.join((0,_common.getAppFolder)(c,platform),'public/appinfo.json');

(0,_common.logTask)("runWebOS:"+platform+":"+target+":"+isHosted,_chalk.default.grey);

cnfg=JSON.parse(_fs.default.readFileSync(configFilePath,'utf-8'));
tId=cnfg.id;
appPath=_path.default.join(tOut,tId+"_"+cnfg.version+"_all.ipk");_context3.t0=


!isHosted;if(!_context3.t0){_context3.next=15;break;}_context3.next=15;return _regenerator.default.awrap((0,_web.buildWeb)(c,platform));case 15:_context3.next=17;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_PACKAGE,"-o "+tOut+" "+tDir+" -n",_common.logTask));case 17:_context3.next=19;return _regenerator.default.awrap(


(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_DEVICE_INFO,'-D'));case 19:devicesResponse=_context3.sent;
devices=parseDevices(devicesResponse);if(!

device){_context3.next=57;break;}

actualDevices=devices.filter(function(d){return d.isDevice;});if(

actualDevices.length){_context3.next=49;break;}_context3.next=26;return _regenerator.default.awrap(

_inquirer.default.prompt([{
type:'confirm',
name:'setupDevice',
message:'Looks like you want to deploy on a device but have none configured. Do you want to configure one?',
default:false}]));case 26:response=_context3.sent;if(!


response.setupDevice){_context3.next=47;break;}

(0,_common.logInfo)('Please follow the instructions from http://webostv.developer.lge.com/develop/app-test/#installDevModeApp on how to setup the TV and the connection with the PC. Then follow the onscreen prompts\n');_context3.next=31;return _regenerator.default.awrap(
(0,_exec.executeAsync)('bash',[c.cli[_constants.CLI_WEBOS_ARES_SETUP_DEVICE]],{stdio:'inherit'}));case 31:_context3.next=33;return _regenerator.default.awrap(

(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_DEVICE_INFO,'-D'));case 33:newDeviceResponse=_context3.sent;
dev=parseDevices(newDeviceResponse);
actualDev=dev.filter(function(d){return d.isDevice;});if(!(

actualDev.length>0)){_context3.next=47;break;}
newDevice=actualDev[0];

(0,_common.logInfo)('Please enter the `Passphrase` from the TV\'s Developer Mode app');_context3.next=41;return _regenerator.default.awrap(
(0,_exec.executeAsync)('bash',[c.cli[_constants.CLI_WEBOS_ARES_NOVACOM],'--device',newDevice.name,'--getkey'],{stdio:'inherit'}));case 41:_context3.next=43;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_INSTALL,"--device "+newDevice.name+" "+appPath,_common.logTask));case 43:_context3.next=45;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_LAUNCH,"--device "+newDevice.name+" "+tId,_common.logTask));case 45:_context3.next=47;break;case 47:_context3.next=55;break;case 49:if(!(





actualDevices.length===1)){_context3.next=55;break;}
tv=actualDevices[0];_context3.next=53;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_INSTALL,"--device "+tv.name+" "+appPath,_common.logTask));case 53:_context3.next=55;return _regenerator.default.awrap(
(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_LAUNCH,"--device "+tv.name+" "+tId,_common.logTask));case 55:_context3.next=79;break;case 57:if(

c.program.target){_context3.next=77;break;}if(!(

devices.length===1)){_context3.next=63;break;}_context3.next=61;return _regenerator.default.awrap(

installAndLaunchApp(c,devices[0].name,appPath,tId));case 61:_context3.next=75;break;case 63:if(!(
devices.length>1)){_context3.next=73;break;}

choices=buildDeviceChoices(devices);_context3.next=67;return _regenerator.default.awrap(
_inquirer.default.prompt([{
name:'chosenDevice',
type:'list',
message:'What device would you like to start the app?',
choices:choices}]));case 67:_response=_context3.sent;if(!

_response.chosenDevice){_context3.next=71;break;}_context3.next=71;return _regenerator.default.awrap(
installAndLaunchApp(c,_response.chosenDevice,appPath,tId));case 71:_context3.next=75;break;case 73:_context3.next=75;return _regenerator.default.awrap(


installAndLaunchApp(c,tSim,appPath,tId));case 75:_context3.next=79;break;case 77:_context3.next=79;return _regenerator.default.awrap(



installAndLaunchApp(c,c.program.target,appPath,tId));case 79:case"end":return _context3.stop();}}});};exports.runWebOS=runWebOS;


























var buildWebOSProject=function buildWebOSProject(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)("buildWebOSProject:"+platform);

var tDir=_path.default.join((0,_common.getAppFolder)(c,platform),'public');
var tOut=_path.default.join((0,_common.getAppFolder)(c,platform),'output');

(0,_web.buildWeb)(c,platform).
then(function(){return(0,_exec.execCLI)(c,_constants.CLI_WEBOS_ARES_PACKAGE,"-o "+tOut+" "+tDir+" -n",_common.logTask);}).
then(function(){
(0,_common.logSuccess)("Your IPK package is located in "+_chalk.default.white(tOut)+" .");
return resolve();
}).
catch(reject);
});};exports.buildWebOSProject=buildWebOSProject;

var configureWebOSProject=function configureWebOSProject(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)('configureWebOSProject');

if(!(0,_common.isPlatformActive)(c,platform,resolve))return;



copyWebOSAssets(c,platform).
then(function(){return(0,_common.copyBuildsFolder)(c,platform);}).
then(function(){return configureProject(c,platform);}).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
});};exports.configureWebOSProject=configureWebOSProject;

var configureProject=function configureProject(c,platform){return new Promise(function(resolve,reject){
(0,_common.logTask)("configureProject:"+platform);

var appFolder=(0,_common.getAppFolder)(c,platform);

var configFile='public/appinfo.json';
(0,_common.writeCleanFile)(_path.default.join((0,_common.getAppTemplateFolder)(c,platform),configFile),_path.default.join(appFolder,configFile),[
{pattern:'{{APPLICATION_ID}}',override:(0,_common.getAppId)(c,platform)},
{pattern:'{{APP_TITLE}}',override:(0,_common.getAppTitle)(c,platform)},
{pattern:'{{APP_VERSION}}',override:_semver.default.coerce((0,_common.getAppVersion)(c,platform))}]);


resolve();
});};