var _interopRequireDefault=require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports,"__esModule",{value:true});exports._updateProfiles=exports.updateProfiles=exports.installCerts=exports.installProfiles=exports.decrypt=exports.encrypt=void 0;var _path=_interopRequireDefault(require("path"));
var _tar=_interopRequireDefault(require("tar"));
var _chalk=_interopRequireDefault(require("chalk"));
var _fs=_interopRequireDefault(require("fs"));
var _common=require("../common");
var _constants=require("../constants");
var _fileutils=require("./fileutils");
var _exec=require("./exec");
var _fastlane=require("../platformTools/apple/fastlane");

var getEnvVar=function getEnvVar(c){
var p1=c.paths.globalConfigFolder.split('/').pop().replace('.','');
var p2=c.files.projectPackage.name.replace('@','').replace('/','_').replace(/-/g,'_');
var envVar=("CRYPTO_"+p1+"_"+p2).toUpperCase();
(0,_common.logDebug)('encrypt looking for env var:',envVar);
return envVar;
};

var encrypt=function encrypt(c){return new Promise(function(resolve,reject){var _c$files$projectConfi,_c$files$projectConfi2,_c$files$projectConfi3;
(0,_common.logTask)('encrypt');

var source="./"+c.files.projectPackage.name;
var destRaw=(_c$files$projectConfi=c.files.projectConfig)==null?void 0:(_c$files$projectConfi2=_c$files$projectConfi.crypto)==null?void 0:(_c$files$projectConfi3=_c$files$projectConfi2.encrypt)==null?void 0:_c$files$projectConfi3.dest;

if(destRaw){
var dest=""+(0,_fileutils.getRealPath)(c,destRaw,'encrypt.dest');
var destTemp=_path.default.join(c.paths.globalConfigFolder,c.files.projectPackage.name.replace('/','-'))+".tgz";

var envVar=getEnvVar(c);
var key=c.program.key||c.process.env[envVar];
if(!key){
reject("encrypt: You must pass "+_chalk.default.white('--key')+" or have env var "+_chalk.default.white(envVar)+" defined");
return;
}
_tar.default.c(
{
gzip:true,
file:destTemp,
cwd:c.paths.globalConfigFolder},

[source]).

then(function(){return(0,_exec.executeAsync)('openssl',[
'enc',
'-aes-256-cbc',
'-salt',
'-in',
destTemp,
'-out',
dest,
'-k',
key],
{privateParams:['-k']});}).
then(function(){
(0,_fileutils.removeFilesSync)([destTemp]);
_fs.default.writeFileSync(dest+".timestamp",new Date().getTime());
(0,_common.logSuccess)("Files succesfully encrypted into "+dest);
resolve();
}).catch(function(e){
reject(e);
});
}else{
(0,_common.logWarning)("You don't have {{ crypto.encrypt.dest }} specificed in "+_chalk.default.white(c.paths.projectConfig));
resolve();
}
});};exports.encrypt=encrypt;

var decrypt=function decrypt(c){return new Promise(function(resolve,reject){var _c$files$projectConfi4,_c$files$projectConfi5,_c$files$projectConfi6;
(0,_common.logTask)('encrypt');

var sourceRaw=(_c$files$projectConfi4=c.files.projectConfig)==null?void 0:(_c$files$projectConfi5=_c$files$projectConfi4.crypto)==null?void 0:(_c$files$projectConfi6=_c$files$projectConfi5.decrypt)==null?void 0:_c$files$projectConfi6.source;

if(sourceRaw){
var source=""+(0,_fileutils.getRealPath)(c,sourceRaw,'decrypt.source');
var ts=source+".timestamp";
var destTemp=_path.default.join(c.paths.globalConfigFolder,c.files.projectPackage.name.replace('/','-'))+".tgz";
var envVar=getEnvVar(c);

var key=c.program.key||c.process.env[envVar];
if(!key){
reject("encrypt: You must pass "+_chalk.default.white('--key')+" or have env var "+_chalk.default.white(envVar)+" defined");
return;
}
(0,_exec.executeAsync)('openssl',[
'enc',
'-aes-256-cbc',
'-d',
'-in',
source,
'-out',
destTemp,
'-k',
key],
{privateParams:['-k']}).
then(function(){
_tar.default.x(
{
file:destTemp,
cwd:c.paths.globalConfigFolder}).

then(function(){
(0,_fileutils.removeFilesSync)([destTemp]);
if(_fs.default.existsSync(ts)){
(0,_fileutils.copyFileSync)(ts,_path.default.join(c.paths.globalConfigFolder,c.files.projectPackage.name,'timestamp'));
}
(0,_common.logSuccess)("Files succesfully extracted into "+c.paths.globalConfigFolder);
resolve();
}).
catch(function(e){return reject(e);});
}).catch(function(e){
reject(e);
});
}else{
(0,_common.logWarning)("You don't have {{ crypto.encrypt.dest }} specificed in "+_chalk.default.white(c.paths.projectConfig));
resolve();
}
});};exports.decrypt=decrypt;

var installProfiles=function installProfiles(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('installProfiles');
if(c.platform!=='ios'){
(0,_common.logError)("installProfiles: platform "+c.platform+" not supported");
resolve();
return;
}

var ppFolder=_path.default.join(c.paths.homeFolder,'Library/MobileDevice/Provisioning Profiles');

if(!_fs.default.existsSync(ppFolder)){
(0,_common.logWarning)("folder "+ppFolder+" does not exist!");
(0,_fileutils.mkdirSync)(ppFolder);
}

var list=(0,_fileutils.getFileListSync)(c.paths.globalProjectFolder);
var mobileprovisionArr=list.filter(function(v){return v.endsWith('.mobileprovision');});

try{
mobileprovisionArr.forEach(function(v){
console.log("installProfiles: Installing: "+v);
(0,_fileutils.copyFileSync)(v,ppFolder);
});
}catch(e){
(0,_common.logError)(e);
}

resolve();
});};exports.installProfiles=installProfiles;

var installCerts=function installCerts(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('installCerts');
if(c.platform!=='ios'){
(0,_common.logError)("_installTempCerts: platform "+c.platform+" not supported");
resolve();
return;
}
var kChain=c.program.keychain||'ios-build.keychain';
var kChainPath=_path.default.join(c.paths.homeFolder,'Library/Keychains',kChain);
var list=(0,_fileutils.getFileListSync)(c.paths.globalProjectFolder);
var cerPromises=[];
var cerArr=list.filter(function(v){return v.endsWith('.cer');});

Promise.all(cerArr.map(function(v){return(0,_exec.executeAsync)('security',[
'import',
v,
'-k',
kChain,
'-A']);})).

then(function(){return resolve();}).
catch(function(e){
(0,_common.logWarning)(e);
resolve();
});
});};exports.installCerts=installCerts;


var updateProfiles=function updateProfiles(c){
(0,_common.logTask)('updateProfiles');
switch(c.platform){
case _constants.IOS:
case _constants.TVOS:
var savedAppConfig=c.files.appConfigFile;
var scheme=c.program.scheme;
var appId=c.appId;
return _updateProfiles(c).
then(function(){
c.files.appConfigFile=savedAppConfig;
c.appId=appId;
c.program.scheme=scheme;
});}

return Promise.reject("updateProfiles: Platform "+c.platform+" not supported");
};exports.updateProfiles=updateProfiles;

var _updateProfiles=function _updateProfiles(c){
(0,_common.logTask)('_updateProfiles',_chalk.default.grey);
var acList=(0,_common.listAppConfigsFoldersSync)(c);
var fullList=[];
acList.forEach(function(v){var _appConfigFile$platfo;
var appConfigFile=(0,_fileutils.readObjectSync)(_path.default.join(c.paths.appConfigsFolder,v,'config.json'));

var buildSchemes=(_appConfigFile$platfo=appConfigFile.platforms[c.platform])==null?void 0:_appConfigFile$platfo.buildSchemes;

Object.keys(buildSchemes).forEach(function(scheme){
fullList.push({
appConfigFile:appConfigFile,
appId:v,
scheme:scheme});

});
});

return fullList.reduce(function(previousPromise,v){return previousPromise.then(function(){return _updateProfile(c,v);});},Promise.resolve());
};exports._updateProfiles=_updateProfiles;

var _updateProfile=function _updateProfile(c,v){return new Promise(function(resolve,reject){
(0,_common.logTask)("_updateProfile:"+v,_chalk.default.grey);
c.files.appConfigFile=v.appConfigFile;
c.program.scheme=v.scheme;
c.appId=v.appId;
(0,_fastlane.updateProfile)(c).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
});};